import java.io.*;
import java.util.Vector;
import java.util.StringTokenizer;

public class MasterClient {
	private static final int numScreens = 9;
	private static PerformerConnection screen[] = new PerformerConnection[numScreens];
	private static int checkTimes = 0;

	public static void main(String[] args) {
		ConnectPerformers();
		
		String[] scripts = FindScripts();
		
		int curScript = 0;
		while (true) {
			System.out.println("Starting: "+scripts[curScript]);
			RunScript(scripts[curScript]);
			

			
		 	curScript++;
		 	if (curScript >= scripts.length) {
				curScript = 0;
				CheckPerformers();
			}
		}
	}
	
	private static void CheckPerformers() {
		checkTimes++;
		if (checkTimes == 10) {
			checkTimes = 0;
			DisconnectPerformers();
			ConnectPerformers();
		}
	}
	
	private static void ConnectPerformers() {
		String thisLine;
		
		
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream("Performers.txt")));
			
			thisLine = in.readLine();
			for (int i=0; i<numScreens; i++) {
				System.out.println("Reading..." + thisLine);
				screen[i] = new PerformerConnection(thisLine);
				new ConnectThread(screen[i]).start();
				thisLine = in.readLine();
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
		
		try {Thread.sleep(8000);} catch (Exception e2) { e2.printStackTrace();}
	}
	
	private static void DisconnectPerformers() {

		for (int i=0; i<numScreens; i++) {
			screen[i].close();
		}
		
	}
	
	private static String[] FindScripts() {
	    File dir = new File("Scripts");
	    FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return !name.startsWith(".") && !name.startsWith("!");
	        }
	    };

		String[] scripts = dir.list(filter);
		
		if (scripts == null) {
	        System.out.println("No scripts were found");
	        System.exit(0);
	    }
	    
	    return scripts;
	}
	
	private static void RunScript(String scriptName) {
		Vector script = new Vector();
		String thisLine,thisToken;
		
		// Load & parse script
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream("Scripts/"+scriptName)));
			while ((thisLine = in.readLine()) != null) {
				if ((thisLine.equals("")) || (thisLine.charAt(0) == '#')) {
					// ignor comments and blank lines
				}
				else {
					StringTokenizer st = new StringTokenizer(thisLine,"\t");
					String[] line = new String[st.countTokens()];
					for (int i = 0; st.hasMoreTokens(); i++) {
						line[i] = st.nextToken();
						if (line[i].equals("")) i--;
					}
					if (line.length > 0) script.add(line);
				}
			}	
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
		int loopCounter[] = new int[10];
		int loopPointer[] = new int[10];
		
		// Run script
		
		for (int pc = 0; pc < script.size(); pc++) {
			String[] line = (String[])(script.get(pc));
			
			System.out.print(pc);
			System.out.print(": ");
			
			try {
				int theScreen = Integer.parseInt(line[0]);
				
				// Since the Integer parsed, we have a content command
				String cmd = line[1];
				for (int x=2; x<line.length; x++) {
					cmd += "	"+line[x];
				}
				System.out.print(theScreen);
				System.out.print(" "+cmd);
				screen[theScreen].Cmd(cmd);
			}
			catch (NumberFormatException e) {
				if (line[0].equals("wait")) {
					System.out.print("Waiting...");
					try {Thread.sleep(Integer.parseInt(line[1]));} catch (Exception e2) { e2.printStackTrace();}
				}
				else if (line[0].equals("do")) {
					try {
						int label = Integer.parseInt(line[1]);
						loopCounter[label] = 1;
						loopPointer[label] = pc;
						System.out.print("Setting Loop Point ");
						System.out.print(label);
					} catch (Exception e2) {}
					
				}
				else if (line[0].equals("loop")) {
					try {
						int label = Integer.parseInt(line[1]);
						int times = Integer.parseInt(line[2]);
						if (loopCounter[label] < times) {
							loopCounter[label]++;
							pc = loopPointer[label];
							System.out.print("Looping ");
							System.out.print(loopCounter[label]);
							System.out.print("/");
							System.out.print(times);
						}
					} catch (Exception e2) {}
				}
			}
			System.out.println("");
		}
	}
	
	
	
	
	
	// Thread that attempts to connect a performer
	public static class ConnectThread extends Thread {
		public PerformerConnection performer = null;

		public ConnectThread(PerformerConnection Pperformer) {
			super("ConnectThread");
			performer = Pperformer;
		}

		public void run() {
			while (!performer.isConnected() && checkTimes <= 8) {
				performer.Connect();
				try {Thread.sleep(30000);} catch (Exception e2) { e2.printStackTrace();}
			}
		}
	}
}
