import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.util.StringTokenizer;





public class PerformerServer {
	private static MyFrame theFrame;
	
	private static String state[];
	
	private static Toolkit tk;
	
	
	private static String imageFilenames[];
	private static Image images[];
	private static Image theImage;
	
	public static void main(String[] args) {
		
		// Set screen default
		state = new String[2];
		state[0] = "w";
		state[1] = "#000000";
		
		// Create screen
		theFrame = new MyFrame();
		theFrame.setVisible(true);
		
		// Start image toolkits
		tk = Toolkit.getDefaultToolkit();

		// Turn off screen blanking
		try {
			Process child = Runtime.getRuntime().exec("setterm -blank 0");
			child = Runtime.getRuntime().exec("xset s off");
			child = Runtime.getRuntime().exec("xset s noblank");
		}
		catch (IOException e) {
			errorState("Couldn't disable screen blanking",e.toString());
		}
		
		// Load Images
		loadImages();
		
		// Start Server
		ServerSocket serverSocket = null;
		boolean listening = true;
		try {
            serverSocket = new ServerSocket(4444);
        } catch (IOException e) {
            errorState("Couldn't listen on port 4444",e.toString());
        }
        
        // Wait for connections and handle them with threads
        PerformerServerThread theServer = null;
		try {
			while (listening) {
				Socket tempSocket = serverSocket.accept();
				if (theServer != null) theServer.stopThread();
				theServer = new PerformerServerThread(tempSocket);
				theServer.start();
			}
		}
		catch (IOException e) {
			errorState("Problem creating server thread",e.toString());
		}
	}
	
	
	
	
	
	
	// Load images
	public static void loadImages() {
		File dir = new File("images");
	    FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return !name.startsWith(".");
	        }
	    };

		imageFilenames = dir.list(filter);
		
		images = new Image[imageFilenames.length];
		 
		for (int i = 0; i < imageFilenames.length; i++) {
			try {
				images[i] = tk.getImage("images/"+imageFilenames[i]);
			}
			catch (Exception e) {e.printStackTrace();}
		}
		
		statusState("Loading Images","");
		
		if (TrackerUtil.waitForImages(images,theFrame)) {
			state = new String[2];
			state[0] = "w";
			state[1] = "#000000";
			theFrame.repaint();
		}
		else {
			errorState("Couldn't load images","");
		}
	}
	
	// Handles state changes
	public static void doState(String pState[]) {
		switch (pState[0].charAt(0)) {
			case 'q':
			System.exit(0);
			break;
			
			case 'r':
			loadImages();
			break;
			
			case 'p':
			for (int i = 0; i < imageFilenames.length; i++) {
				if (imageFilenames[i].equals(pState[1])) {
					state = pState;
					theImage = images[i];
					theFrame.thePanel.repaint();
					break;
				}
			}
			break;
			
			default:
			state = pState;
			theFrame.thePanel.repaint();
			break;
		}
	}
	
	// Reports errors through the 'text' state
	public static void errorState(String errorMsg,String errorMsg2) {
		String newState[] = new String[9];
		newState[0] = "t";
		newState[1] = "sans-serif";
		newState[2] = "14";
		newState[3] = "#000000";
		newState[4] = "#FF0000";
		newState[5] = "3";
		newState[6] = "* Error *";
		newState[7] = errorMsg;
		newState[8] = errorMsg2;
		
		doState(newState);
	}
	
	// Reports errors through the 'text' state
	public static void statusState(String errorMsg,String errorMsg2) {
		String newState[] = new String[9];
		newState[0] = "t";
		newState[1] = "sans-serif";
		newState[2] = "14";
		newState[3] = "#000000";
		newState[4] = "#999999";
		newState[5] = "2";
		newState[6] = errorMsg;
		newState[7] = errorMsg2;
		
		doState(newState);
	}

	
	
	
	
	
	
	
	
	
	// Thread that handles communication with the Master computer
	public static class PerformerServerThread extends Thread {
		private Socket socket = null;
		private boolean threadIsRunning;

		public PerformerServerThread(Socket socket) {
			super("PerformerServerThread");
			this.socket = socket;
		}

		public void run() {
			threadIsRunning = true;
			
			try {
				PrintWriter out = new PrintWriter(socket.getOutputStream(), true);
				BufferedReader in = new BufferedReader(new InputStreamReader(socket.getInputStream()));

				String inputLine;

				while (((inputLine = in.readLine()) != null) && threadIsRunning) {   
					StringTokenizer toks = new StringTokenizer(inputLine,"\t");
					int numOfParms = toks.countTokens();
					String newState[] = new String[numOfParms];

					for (int i=0;i<numOfParms;i++)
						newState[i] = toks.nextToken();
						
					doState(newState);
				}
				out.close();
				in.close();
				socket.close();

			} catch (IOException e) {
				errorState("Server Error",e.toString());
			}
		}
		
		public void stopThread() {
			threadIsRunning = false;
		}
	}
	
	
	
	

	// Frame which draws states to the screen
	public static class MyFrame extends JFrame {
		public MyPanel thePanel;
		
		public MyFrame() {
			super();
			setBounds(0,0, 640, 500);
		
		 	thePanel = new MyPanel();
		
			setContentPane(thePanel);
		}
		
		public class MyPanel extends JPanel {
			public void paintComponent(Graphics g) {
				super.paintComponent(g);
				
				switch (state[0].charAt(0)) {
				
					case 'b':
					g.setColor(Color.decode(state[1]));
					g.fillRect(0,0,640,480);
					g.setColor(Color.decode(state[2]));
					g.fillRect(96,87,448,306);
					g.setColor(Color.decode(state[3]));
					g.fillRect(204,188,232,104);
					break;
					
					case 'v':
					g.setColor(Color.decode(state[1]));
					g.fillRect(0,0,213,480);
					g.setColor(Color.decode(state[2]));
					g.fillRect(213,0,214,480);
					g.setColor(Color.decode(state[3]));
					g.fillRect(427,0,213,480);
					break;
					
					case 'h':
					g.setColor(Color.decode(state[1]));
					g.fillRect(0,0,640,160);
					g.setColor(Color.decode(state[2]));
					g.fillRect(0,160,640,160);
					g.setColor(Color.decode(state[3]));
					g.fillRect(0,320,640,160);
					break;
					
					case 't':
					g.setColor(Color.decode(state[3]));
					g.fillRect(0,0,640,480);
					g.setFont(new Font(state[1],Font.PLAIN,Integer.parseInt(state[2])));
					g.setColor(Color.decode(state[4]));
					FontMetrics fMet = g.getFontMetrics();
					int stringheight = fMet.getHeight();
					int lines = Integer.parseInt(state[5]);
					for (int i=0; i<lines; i++) {
						int stringWidth = fMet.stringWidth(state[6+i]);
						int vert = (480-stringheight*lines/2)/2+stringheight*i;
						g.drawString(state[6+i],(640-stringWidth)/2,vert);
					}
					break;
					
					case 'w':
					g.setColor(Color.decode(state[1]));
					g.fillRect(0,0,640,480);
					break;
					
					case 'p':
						g.drawImage(theImage,0,0,this);
					break;
					
					default:
						errorState("Unknown Command: "+state[0],"");
					break;
				}
			}
		}
	}
}
